"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.Plan = void 0;
const eventemitter3_1 = __importDefault(require("eventemitter3"));
class Plan {
    constructor(plan) {
        var _a, _b;
        this.executing = false;
        this.executed = false;
        this.promise = plan.promise;
        this.steps = (_a = plan.steps) !== null && _a !== void 0 ? _a : [];
        this.eventEmitter = (_b = plan.eventEmitter) !== null && _b !== void 0 ? _b : new eventemitter3_1.default();
    }
    static make() {
        return new Plan({
            promise: (initialState) => __awaiter(this, void 0, void 0, function* () { return initialState; }),
        });
    }
    addStep(step) {
        const { newStep, handler } = this.parseInputStep(step);
        const promise = (initialState, steps) => __awaiter(this, void 0, void 0, function* () {
            let state;
            try {
                state = yield this.promise(initialState, steps);
            }
            catch (error) {
                this.changeStepStatus(steps, newStep, 'canceled');
                throw error;
            }
            return this.processStep(steps, state, newStep, handler);
        });
        return new Plan({
            promise,
            steps: [...this.steps, newStep],
            eventEmitter: this.eventEmitter,
        });
    }
    prependStep(step) {
        const { newStep, handler } = this.parseInputStep(step);
        const promise = (newInitialState, steps) => __awaiter(this, void 0, void 0, function* () {
            let initialState;
            try {
                initialState = yield this.processStep(steps, newInitialState, newStep, handler);
            }
            catch (error) {
                this.steps.forEach((step) => this.changeStepStatus(steps, step, 'canceled'));
                throw error;
            }
            return this.promise(initialState, steps);
        });
        return new Plan({
            promise,
            steps: [newStep, ...this.steps],
            eventEmitter: this.eventEmitter,
        });
    }
    parseInputStep(step) {
        var _a, _b;
        const { handler } = step;
        const newStep = {
            name: step.name,
            status: 'pending',
            hidden: (_a = step.hidden) !== null && _a !== void 0 ? _a : false,
            optional: (_b = step.optional) !== null && _b !== void 0 ? _b : false,
            onError: step.onError,
        };
        return { newStep, handler };
    }
    processStep(steps, from, step, handler) {
        var _a;
        return __awaiter(this, void 0, void 0, function* () {
            this.changeStepStatus(steps, step, 'running');
            try {
                const to = yield handler(from);
                this.changeStepStatus(steps, step, 'successful');
                return to;
            }
            catch (error) {
                this.changeStepStatus(steps, step, 'failed');
                (_a = step.onError) === null || _a === void 0 ? void 0 : _a.call(step, error);
                if (step.optional) {
                    // If a step is optional, it's destination state should match
                    // the source state. Otherwise, steps cannot be composed.
                    return from;
                }
                else {
                    throw error;
                }
            }
        });
    }
    onChange(listener) {
        this.eventEmitter.addListener('change', listener);
        return this;
    }
    notifyChange(step, steps) {
        this.eventEmitter.emit('change', step, steps);
    }
    changeStepStatus(steps, step, newStatus) {
        step.status = newStatus;
        this.notifyChange(step, steps);
    }
    getSteps() {
        return this.steps;
    }
    getVisibleSteps() {
        return this.steps.filter((step) => !step.hidden);
    }
    execute(initialState) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                this.executing = true;
                this.executed = false;
                const state = initialState !== null && initialState !== void 0 ? initialState : undefined;
                return yield this.promise(state, this.steps);
            }
            finally {
                this.executing = false;
                this.executed = true;
            }
        });
    }
}
exports.Plan = Plan;
//# sourceMappingURL=Plan.js.map