import { ModuleClient } from "../../shared";
import { UploadMetadataOperation, PlanUploadMetadataOperation, CreateNftOperation, FindNftByMintOperation, UpdateNftOperation, } from './operations';
export class NftClient extends ModuleClient {
    async uploadMetadata(input, confirmOptions) {
        return this.metaplex.execute(new UploadMetadataOperation(input), confirmOptions);
    }
    async planUploadMetadata(input, confirmOptions) {
        return this.metaplex.execute(new PlanUploadMetadataOperation(input), confirmOptions);
    }
    async createNft(input, confirmOptions) {
        const operation = new CreateNftOperation(input);
        const createNftOutput = await this.metaplex.execute(operation, confirmOptions);
        const nft = await this.findNft({ mint: createNftOutput.mint.publicKey });
        return { ...createNftOutput, nft };
    }
    async findNft({ mint }) {
        return this.metaplex.execute(new FindNftByMintOperation(mint));
    }
    async updateNft(nft, input, confirmOptions) {
        const operation = new UpdateNftOperation({ ...input, nft });
        const updateNftOutput = await this.metaplex.execute(operation, confirmOptions);
        const updatedNft = await this.findNft({ mint: nft.mint });
        return { ...updateNftOutput, nft: updatedNft };
    }
}
//# sourceMappingURL=NftClient.js.map