import EventEmitter from 'eventemitter3';
declare type StepStatus = 'pending' | 'running' | 'successful' | 'failed' | 'canceled';
interface Step {
    name: string;
    status: StepStatus;
    hidden: boolean;
    optional: boolean;
    onError?: (error: unknown) => void;
}
export declare type InputStep<From, To> = Pick<Step, 'name'> & Partial<Omit<Step, 'status'>> & {
    handler: (state: From) => Promise<To>;
};
export declare class Plan<I, O> {
    readonly promise: (state: I, steps: Step[]) => Promise<O>;
    readonly steps: Step[];
    readonly eventEmitter: EventEmitter;
    executing: boolean;
    executed: boolean;
    private constructor();
    static make<T = undefined>(): Plan<T, T>;
    addStep<T>(step: InputStep<O, T>): Plan<I, T>;
    prependStep<T>(step: InputStep<T, I>): Plan<T, O>;
    private parseInputStep;
    private processStep;
    onChange(listener: (step: Step, steps: Step[]) => void): this;
    private notifyChange;
    private changeStepStatus;
    getSteps(): Step[];
    getVisibleSteps(): Step[];
    execute(initialState?: I): Promise<O>;
}
export {};
